/*   PROGRAMM BERCHNET DIE SPHRISCHEN BESSELFUNKTIONEN 1. UND 2. ART   */
/*   DER ORDNUNG l. DIESE WERDEN IN DER STRUKTUR ZYLFUNKT INFOLGENDER   */
/*   REIHENFOLGE ABGELEGT :   -sph.Besselfunktion,                      */
/*                            -deren erste Ableitung,                   */
/*        		      -sph.Neumannfunktion,                     */
/*			      -deren erste Ableitung.                   */

#ifndef __BESSEL
#define __BESSEL

/* #define __TEST_BESSEL */            /* Beim Ausfhren nur dieser     */
				       /* Funktion Kommentare entfernen */
#include "quer.h"

struct ZYLFUNKT m_sphbes (TYP x, int l)
				     /* Berechnung der Besselfunktionen */
{				     /* mit dem Rekursionsvervahren 1   */
int i;
struct ZYLFUNKT ergeb;
TYP besel1,besel2,besel3;                /* Zwischenwerte bei Rekursion */
TYP zwi;
if (l==0) {                            /* analytische Ausdrcke fr l=0 */
	   ergeb.bessel=sin(x)/x;
	   ergeb.besabl=cos(x)/x-(sin(x)/(x*x));
	   ergeb.neuman=-cos(x)/x;
	   ergeb.neuabl=sin(x)/x+cos(x)/(x*x);
	   }
if (l==1) {                            /* analytische Ausdrcke fr l=1 */
	   ergeb.bessel=sin(x)/(x*x)-cos(x)/x;
	   ergeb.besabl=2.*cos(x)/(x*x)+(1./x-2./(x*x*x))*sin(x);
	   ergeb.neuman=-cos(x)/(x*x)-sin(x)/x;
	   ergeb.neuabl=2.*sin(x)/(x*x)-(1./x-2./(x*x*x))*cos(x);
	  }
if (l>1)
 {                                 /* Brechnung der sph. Besselfunktion */
  besel1=sin(x)/x;                     /* analytischer Ausdruck fr l=0 */
  besel2=sin(x)/(x*x)-cos(x)/x;        /* analytischer Ausdruck fr l=1 */
  for (i=1;i<l;i++)
  {
  besel3=((2.*i+1.)/x)*besel2-besel1;               /* Rekursionsformel */
  besel1=besel2;
  besel2=besel3;
  }
  ergeb.bessel=besel3;
					    /* Berechnung der Ableitung */
  besel3=((2.*l+1.)/x)*besel2-besel1;               /* Rekursionsformel */
  zwi=(1./(2.*l+1.))*(l*besel1-(l+1.)*besel3);/* Formel fr d.Ableitung */
  ergeb.besabl=zwi;
				 /* Berechnung der sph. Neumannfunktion */
  besel1=-cos(x)/x;                    /* analytischer Ausdruck fr l=0 */
  besel2=-cos(x)/(x*x)-sin(x)/x;       /* analytischer Ausdruck fr l=1 */
  for (i=1;i<l;i++)
  {
  besel3=((2.*i+1.)/x)*besel2-besel1;               /* Rekursionsformel */
  besel1=besel2;
  besel2=besel3;
  }
  ergeb.neuman=besel3;
					    /* Berechnung der Ableitung */
  besel3=((2.*l+1.)/x)*besel2-besel1;               /* Rekursionsformel */
  zwi=1./(2.*l+1.)*(l*besel1-(l+1.)*besel3);  /* Formel fr d.Ableitung */
  ergeb.neuabl=zwi;

 }
return ergeb;
}

/* Funktionen aus " NUMERICAL RECIPES " */

#include <stddef.h>
#include "utilites.c"

#define ANSI                                        /* ANSI Ddefiniert */
#if defined(__STDC__) || defined(ANSI) || defined(NRANSI)       /* ANSI */

#define NR_END 1
#define FREE_ARG char*

TYP chebev(TYP a, TYP b, TYP c[],int m, TYP x)
{
	void nrerror(char error_text[]);
	TYP d=0.0,dd=0.0,sv,y,y2;
	int j;

	if ((x-a)*(x-b) > 0.0) nrerror("x not in range in routine chebev");
	y2=2.0*(y=(2.0*x-a-b)/(b-a));
	for (j=m-1;j>=1;j--) {
		sv=d;
		d=y2*d-dd+c[j];
		dd=sv;
	}
	return y*d-dd+0.5*c[0];
}
/* (C) Copr. 1986-92 Numerical Recipes Software $1&. */


#define NUSE1 5
#define NUSE2 5

void beschb(TYP x, TYP *gam1, TYP *gam2, TYP *gampl,TYP *gammi)
{
	TYP chebev(TYP a, TYP b,
			  TYP c[], int m, TYP x);
	TYP xx;
	static TYP c1[] = {
		-1.142022680371172e0,6.516511267076e-3,
		3.08709017308e-4,-3.470626964e-6,6.943764e-9,
		3.6780e-11,-1.36e-13};
	static TYP c2[] = {
		1.843740587300906e0,-0.076852840844786e0,
		1.271927136655e-3,-4.971736704e-6,-3.3126120e-8,
		2.42310e-10,-1.70e-13,-1.0e-15};

	xx=8.0*x*x-1.0;
	*gam1=chebev(-1.0,1.0,c1,NUSE1,xx);
	*gam2=chebev(-1.0,1.0,c2,NUSE2,xx);
	*gampl= *gam2-x*(*gam1);
	*gammi= *gam2+x*(*gam1);
}
#undef NUSE1
#undef NUSE2
/* (C) Copr. 1986-92 Numerical Recipes Software $1&. */

#define NRANSI
#define EPS 1.0E-16
#define FPMIN 1.0E-30
#define MAXIT 10000
#define XMIN 2.0

/* Deklarationen aus "nrutil.h" */
#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
static int imaxarg1,imaxarg2;
#define IMAX(a,b) (imaxarg1=(a),imaxarg2=(b),(imaxarg1) > (imaxarg2) ?\
	(imaxarg1) : (imaxarg2))


void bessjy(TYP x, TYP xnu, TYP *rj, TYP *ry, TYP *rjp, TYP *ryp)
{
	void beschb(TYP x, TYP *gam1, TYP *gam2, TYP *gampl,
		TYP *gammi);
	int i,isign,l,nl;
	TYP a,b,br,bi,c,cr,ci,d,del,del1,den,di,dlr,dli,dr,e,f,fact,fact2,
		fact3,ff,gam,gam1,gam2,gammi,gampl,h,p,pimu,pimu2,q,r,rjl,
		rjl1,rjmu,rjp1,rjpl,rjtemp,ry1,rymu,rymup,rytemp,sum,sum1,
		temp,w,x2,xi,xi2,xmu,xmu2;

	if (x <= 0.0 || xnu < 0.0) nrerror("bad arguments in bessjy");
	nl=(x < XMIN ? (int)(xnu+0.5) : IMAX(0,(int)(xnu-x+1.5)));
	xmu=xnu-nl;
	xmu2=xmu*xmu;
	xi=1.0/x;
	xi2=2.0*xi;
	w=xi2/MM_PI;
	isign=1;
	h=xnu*xi;
	if (h < FPMIN) h=FPMIN;
	b=xi2*xnu;
	d=0.0;
	c=h;
	for (i=1;i<=MAXIT;i++) {
		b += xi2;
		d=b-d;
		if (fabs(d) < FPMIN) d=FPMIN;
		c=b-1.0/c;
		if (fabs(c) < FPMIN) c=FPMIN;
		d=1.0/d;
		del=c*d;
		h=del*h;
		if (d < 0.0) isign = -isign;
		if (fabs(del-1.0) < EPS) break;
	}
	if (i > MAXIT) nrerror("x too large in bessjy; try asymptotic expansion");
	rjl=isign*FPMIN;
	rjpl=h*rjl;
	rjl1=rjl;
	rjp1=rjpl;
	fact=xnu*xi;
	for (l=nl;l>=1;l--) {
		rjtemp=fact*rjl+rjpl;
		fact -= xi;
		rjpl=fact*rjtemp-rjl;
		rjl=rjtemp;
	}
	if (rjl == 0.0) rjl=EPS;
	f=rjpl/rjl;
	if (x < XMIN) {
		x2=0.5*x;
		pimu=MM_PI*xmu;
		fact = (fabs(pimu) < EPS ? 1.0 : pimu/sin(pimu));
		d = -log(x2);
		e=xmu*d;
		fact2 = (fabs(e) < EPS ? 1.0 : sinh(e)/e);
		beschb(xmu,&gam1,&gam2,&gampl,&gammi);
		ff=2.0/MM_PI*fact*(gam1*cosh(e)+gam2*fact2*d);
		e=exp(e);
		p=e/(gampl*MM_PI);
		q=1.0/(e*MM_PI*gammi);
		pimu2=0.5*pimu;
		fact3 = (fabs(pimu2) < EPS ? 1.0 : sin(pimu2)/pimu2);
		r=MM_PI*pimu2*fact3*fact3;
		c=1.0;
		d = -x2*x2;
		sum=ff+r*q;
		sum1=p;
		for (i=1;i<=MAXIT;i++) {
			ff=(i*ff+p+q)/(i*i-xmu2);
			c *= (d/i);
			p /= (i-xmu);
			q /= (i+xmu);
			del=c*(ff+r*q);
			sum += del;
			del1=c*p-i*del;
			sum1 += del1;
			if (fabs(del) < (1.0+fabs(sum))*EPS) break;
		}
		if (i > MAXIT) nrerror("bessy series failed to converge");
		rymu = -sum;
		ry1 = -sum1*xi2;
		rymup=xmu*xi*rymu-ry1;
		rjmu=w/(rymup-f*rymu);
	} else {
		a=0.25-xmu2;
		p = -0.5*xi;
		q=1.0;
		br=2.0*x;
		bi=2.0;
		fact=a*xi/(p*p+q*q);
		cr=br+q*fact;
		ci=bi+p*fact;
		den=br*br+bi*bi;
		dr=br/den;
		di = -bi/den;
		dlr=cr*dr-ci*di;
		dli=cr*di+ci*dr;
		temp=p*dlr-q*dli;
		q=p*dli+q*dlr;
		p=temp;
		for (i=2;i<=MAXIT;i++) {
			a += 2*(i-1);
			bi += 2.0;
			dr=a*dr+br;
			di=a*di+bi;
			if (fabs(dr)+fabs(di) < FPMIN) dr=FPMIN;
			fact=a/(cr*cr+ci*ci);
			cr=br+cr*fact;
			ci=bi-ci*fact;
			if (fabs(cr)+fabs(ci) < FPMIN) cr=FPMIN;
			den=dr*dr+di*di;
			dr /= den;
			di /= -den;
			dlr=cr*dr-ci*di;
			dli=cr*di+ci*dr;
			temp=p*dlr-q*dli;
			q=p*dli+q*dlr;
			p=temp;
			if (fabs(dlr-1.0)+fabs(dli) < EPS) break;
		}
		if (i > MAXIT) nrerror("cf2 failed in bessjy");
		gam=(p-f)/q;
		rjmu=sqrt(w/((p-f)*gam+q));
		rjmu=SIGN(rjmu,rjl);
		rymu=rjmu*gam;
		rymup=rymu*(p+q/gam);
		ry1=xmu*xi*rymu-rymup;
	}
	fact=rjmu/rjl;
	*rj=rjl1*fact;
	*rjp=rjp1*fact;
	for (i=1;i<=nl;i++) {
		rytemp=(xmu+i)*xi2*ry1-rymu;
		rymu=ry1;
		ry1=rytemp;
	}
	*ry=rymu;
	*ryp=xnu*xi*rymu-ry1;
}
#undef EPS
#undef FPMIN
#undef MAXIT
#undef XMIN
#undef PI
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software $1&. */

#define RTPIO2 1.2533141

struct ZYLFUNKT sphbes(TYP x, int n)
{
	void bessjy(TYP x, TYP xnu, TYP *rj,
		  TYP *ry, TYP *rjp,TYP *ryp);
	void nrerror(char error_text[]);
	struct ZYLFUNKT er;
	TYP factor,order,rj,rjp,ry,ryp;

	if (n < 0 || x <= 0.0) nrerror("bad arguments in sphbes");
	order=n+0.5;
	bessjy(x,order,&rj,&ry,&rjp,&ryp);
	factor=RTPIO2/sqrt(x);
	er.bessel=factor*rj;
	er.neuman=factor*ry;
	er.besabl=factor*rjp-er.bessel/(2.0*x);
	er.neuabl=factor*ryp-er.neuman/(2.0*x);
	return er;
}
#undef RTPIO2

/* Ende der Funktionen aus " NUMERICAL RECIPES " */


struct ZYLFUNKT bessel (TYP x, int l)
{
struct ZYLFUNKT erg;          /* Auswahl welches Verfahren genutzt wird */

if (x<1.) { erg=sphbes(x,l); return erg; }
if (x<5.) { if (l<6) erg=m_sphbes(x,l);
	    else erg=sphbes(x,l);
	    return erg; }
if (x<10.) { if (l<11) erg=m_sphbes(x,l);
	    else erg=sphbes(x,l);
	    return erg; }
else       { if (l<21) erg=m_sphbes(x,l);
	    else erg=sphbes(x,l);
	    return erg; }
}

#ifdef BESSEL_TEST

int main (void)
{
int l=100;
TYP x=0.1;
struct ZYLFUNKT erg;
						/* Ausgabe der Werte */
erg=bessel(x,l);
printf("\n\n\n\n\n\n\n\n");
printf("     bessel : %+.10le bei x= %le mit l= %d \n",erg.bessel,x,l);
printf("     besabl : %+.10le bei x= %le mit l= %d \n",erg.besabl,x,l);
printf("     neuman : %+.10le bei x= %le mit l= %d \n",erg.neuman,x,l);
printf("     neuabl : %+.10le bei x= %le mit l= %d \n",erg.neuabl,x,l);
getchar();
getchar();
return 0;
}

#endif
#endif
#endif
