!******************************************************************************
!******************************************************************************
!******************************************************************************
module composition

use constants
use fermis
implicit none
save

  double precision:: kbt,thermlambda,kappa,pol,rpol,metomi,mue_e_id,bca
  double precision, pointer:: ne
  double precision, dimension(:),allocatable::spinparameter,energy,radius
  double precision, dimension(:),allocatable::i_array,eta0

  integer:: Z_min, Z_max, minion
  integer:: stelle, maxion, functyp
  logical:: tab_hs, control1

contains 

!************** System of Equations ***********************
subroutine mk_k(n,beta_k)
!is funktion for nag-Routine c05nbf?
!in- and output
double precision,dimension(maxion+2),intent(in)::n
double precision,dimension(maxion),intent(out)::beta_k

!rest
double precision,dimension(-1:maxion)::ns,ns_hilf,mue_array
double precision,dimension(-1:maxion)::diff_n,faktor
double precision,dimension(1:3)::fs0
double precision,dimension(-1:maxion,1:3)::fs,mue_arr
double precision,dimension(1:maxion)::k
double precision hilf, fs0a
integer i,j
logical nohalt

if (control1) print*, 'in mk_k'

!initializing
  mue_arr=0.0d0
  mue_array=0.0d0
  fs0=0.0d0
  fs=0.0d0

  ns=0.0d0
  ns(-1:maxion)=n(1:maxion+2)

  !hard sphere contributions
  if (tab_hs) then
     if (control1) print*, 'mk_k: hard sphere contributions'

     !start
     call mkf_hs_forst(ns,fs0(1))

     !intervall for derivations (>8 digital numbers)
     faktor=1.0d-6
     diff_n=faktor*ns

     do i=-1,Z_max
        if (i==-1 .or. i>=Z_min) then
           nohalt=.true.
           ns_hilf=ns
           do while (nohalt .and. faktor(i)<1.0d0)
              ns_hilf(i)=ns(i)+diff_n(i)
              call mkf_hs_forst(ns_hilf,fs(i,1))
              if (dabs(1.0d0-fs(i,1)/fs0(1))>=1.0d-9) then
                 nohalt=.false.
              else
                 faktor(i)=1.0d1*faktor(i)
                 diff_n(i)=faktor(i)*ns(i)
              end if
           end do
           if (faktor(i)/=1.0d0) then
              mue_arr(i,1)=(fs(i,1)-fs0(1))/diff_n(i)
           else
              faktor(i)=1.0d1
              fs0a=fs0(1)
              do while(dabs(1.0d0-fs(i,1)/fs0a)<1.0d-9)
                 ns_hilf(i)=ns_hilf(i)*faktor(i)
                 call mkf_hs_forst(ns_hilf,fs0a)
                 call mkf_hs_forst(1.1d0*ns_hilf,fs(i,1))
              end do
              mue_arr(i,1)=(fs(i,1)-fs0a)/0.1d0*ns_hilf(i)
           end if
        end if
     end do
     if (control1) print*, 'f_hs ready'
  end if

  !Coulomb contributions
  if (control1) print*, 'Coulomb contributions'

  !start
  call pade(ns,fs0(2))

  !intervall for derivations (>8 digital numbers)
  faktor=1.0d-6
  diff_n=faktor*ns
  do i=-1,maxion
     if (i==-1 .or. i>=minion) then
        nohalt=.true.
        ns_hilf=ns
        do while (nohalt .and. faktor(i)<1.0d0)
           ns_hilf(i)=ns(i)+diff_n(i)
           call pade(ns_hilf,fs(i,2))
           if (dabs(1.0d0-fs(i,2)/fs0(2))>=1.0d-9) then
              nohalt=.false.
           else
              faktor(i)=1.0d1*faktor(i)
              diff_n(i)=faktor(i)*ns(i)
           end if
        end do
        if (faktor(i)/=1.0d0) then
           mue_arr(i,2)=(fs(i,2)-fs0(2))/diff_n(i)
        else
           faktor(i)=1.0d1
           fs0a=fs0(2)
           do while(dabs(1.0d0-fs(i,2)/fs0a)<1.0d-9)
              ns_hilf(i)=ns_hilf(i)*faktor(i)
              call pade(ns_hilf,fs0a)
              call pade(1.1d0*ns_hilf,fs(i,2))
           end do
           mue_arr(i,2)=(fs(i,2)-fs0a)/0.1d0*ns_hilf(i)
        end if
     end if
  end do

  !ea-WW
  if (Z_min==0) then
     if (control1) print*, 'e-a contributions'
     mue_arr(-1,3)=ns(0)*bca
     mue_arr(0,3)=ns(-1)*bca
  end if

  !chemical potentials
  if (control1) print*, 'chemical potentials'
  mue_array(-1:Z_max)=&
!       &mue_arr(:,1)+mue_arr(:,2)+mue_arr(:,3)
      &mue_arr(-1:Z_max,1)+mue_arr(-1:Z_max,2)+mue_arr(-1:Z_max,3)
  mue_array(-1)=mue_array(-1)+mue_e_id

  !determination of k and beta_k
  if (control1) print*, 'determination of k and beta_k'
  k(Z_min+1:Z_max)=energy(Z_min+1:Z_max)
  k(Z_min+1:Z_max)=k(Z_min+1:Z_max)&
                 &+mue_array(Z_min+1:Z_max)-mue_array(Z_min:Z_max-1)&
                 &+mue_array(-1)
  beta_k(Z_min+1:Z_max)=k(Z_min+1:Z_max)/kbt
  if (control1) print*, 'out of mk_k'

end subroutine mk_k


!****************** free energies **************************

!*******hard sphere part ******

subroutine mkf_hs_forst(n,fhs_forst)
!based on Mansoori's mixing rule for 
!compacted by Foerster
!in- and output
  double precision, dimension(maxion+2), intent(in)::n
  double precision, intent(out)::fhs_forst

!coefficients and helping variables
  double precision, dimension(-1:maxion):: ns, x
  double precision, dimension(3)::forst
  double precision, dimension(0:maxion):: pack,d1_vec,d2_vec,d3_vec
  double precision  sum_eta, eta1,d1,d2,d3, ntot, fak1, fak2
  integer i,j

  ns=0
  ns(-1:maxion)=n(1:maxion+2)
  forst=0.0d0
  pack=eta0(0:maxion)*ns(0:maxion) !electron contributions = 0
  sum_eta=sum(pack)
  ntot=sum(ns) !ions+electrons
  x=ns/ntot

  d1_vec=2.0d0*radius*x(0:maxion)
  d2_vec=d1_vec*radius*2.0d0
  d3_vec=d2_vec*radius*2.0d0

  d1=sum(d1_vec)
  d2=sum(d2_vec)
  d3=sum(d3_vec)
  fak1=d2**3/d3**2
  fak2=d2*d1/d3

  eta1=1.0d0-sum_eta
  forst(1)=fak1*sum_eta/eta1**2
  forst(2)=3.0d0*fak2*sum_eta/eta1
  forst(3)=(fak1-1.0d0)*dlog(eta1)

  fhs_forst=ntot*kbt*sum(forst)

end subroutine mkf_hs_forst

!*********** charged particles **********************
subroutine pade(n,f_pade)
implicit none
!in- and output
  double precision, dimension(maxion+2), intent(in)::n
  double precision, intent(out)::f_pade

!helping variables
  double precision, dimension(-1:maxion),target::ns
  double precision, dimension(:),allocatable::beta
  double precision, dimension(:),pointer::n_i
  double precision, dimension(5)::qu
  double precision tau,eps
  double precision zie,nie,zii,nii,zee,nee,p0,p1,fe0,fe1
  double precision fee,fie,fii
  double precision nt,rs,drs,nl,nd,ni,zeff,z1,z2,z3,z53,zlog,e1,e2

!initialization
  if (control1) print*, 'initialization'
  ns(-1:maxion)=n(1:maxion+2)
  n_i=>ns(minion:Z_max)
  ni=sum(n_i)

  allocate(beta(minion:Z_max))
  beta=ns(minion:Z_max)/ni
  zeff=ns(-1)/ni
  z1=sum(i_array(minion:Z_max)*n_i)/ni
  z2=sum(i_array(minion:Z_max)**2*n_i)/ni
  z3=sum(i_array(minion:Z_max)**3*n_i)/ni
  z53=sum(i_array(minion:Z_max)**(5.0/3.0)*n_i)/ni
  zlog=4.0d0*sum(i_array(minion:Z_max)**3*n_i*beta*dlog(i_array(minion:Z_max)))

  nt=thermlambda**3.0d0*ns(-1)
  rs=(0.75d0/pi/ns(-1))**(1.0d0/3.0d0)
  drs=dsqrt(rs)

!e-e contribution
  if (control1) print*, 'e-e contribution'
  tau=(kbt/pi)**0.25d0
  eps=9.163d-1/rs&
     +1.244d-1*dlog((1.0d0+3.008d-1*drs+2.117d0/drs)/(1.0d0+3.008d-1*drs))
  fe0=(2.0d0/3.0d0)*tau
  fe1=(1.76776695d-1+2.5d-1*tau*tau-0.88388347d-1*kbt)/fe0
  zee=fe0*dsqrt(nt)+2.5d-1*tau*tau*nt+3.0d0*eps*nt*nt
  nee=1.0d0+3.0d0*nt*nt+fe1*dsqrt(nt)
  fee=-ns(-1)*zee/nee

!missing Coulomb contributions
  if (control1) print*, 'rest of Coulomb contribution'
  nl=ni*8.0d0/kbt**3.0
  nd=nl*z1*z53**3.0
  p0=1.1816359d0*z2**1.5d0
  p1=0.696041d0*dsqrt(2.0d0*metomi)*z2*z2/p0
  qu(2)=1.1816359d0*((zeff+z2)**1.5d0-zeff**1.5d0-z2**1.5D0) 
  !qu(2) is Q_0 in [Foerster 90]
  qu(1)=(2.784164d0*zeff*z2-p1*p0)/qu(2)
  qu(3)=0.5236d0*(z3-zeff)*(zlog+(z3-zeff)*dlog(2.909d1*(z2+zeff)))/qu(2)
  qu(4)=1.0472d0*(z3-zeff)**2.0d0/qu(2)
  if (minion==1 .and. beta(minion)==1.0d0) then
        qu(5)=-0.5236d0/1.0472d0*(2.0d0+dlog(2.909d1*(z2+zeff)))
        qu(5)=dexp(qu(5))
  else
     !qu(5)=dexp(-qu(3)/qu(4))
     qu(5)=-0.5236d0/1.0472d0*(zlog/(z3-zeff)+dlog(2.909d1*(z2+zeff)))
     qu(5)=dexp(qu(5))
  end if
  e1=1.4474d0*nd**(1.0/3.0)-4.2944d0*nd**(1.0/12.0)+0.6712d0/nd**(1.0/12.0)&
    +(0.2726d0*dlog(nd)+2.983d0)
  e2=0.8511d0*rs*nd**(1.0/12.0)/(1.0d0+0.3135d0/nd**(1.0/12.0))&
    /(1.0d0+1.137d0*rs*dsqrt(kbt))&
    +rs*nd**(1.0/3.0)*(0.0726d0+0.0161d0*rs)/(1.0d0+0.0887d0*rs*rs)
  deallocate(beta)

 !i-e contribution
  if (control1) print*, 'i-e contribution'
  zie=qu(2)*dsqrt(nl)+nt**1.5d0*e2
  nie=1.0d0+qu(1)*dsqrt(nl*kbt)+nt**1.5d0&
     +qu(4)*dsqrt(nl)*dlog(1.0d0+qu(5)/dsqrt(nl))
  fie=-ni*kbt*zie/nie

 !i-i contribution
  if (control1) print*, 'i-i contribution'
  zii=p0*dsqrt(nl)+1.0d3*nd**1.5d0*e1
  nii=1.0d0+p1*dsqrt(nl*kbt)+1.0d3*nd**1.5d0
  fii=-ni*kbt*zii/nii

 !summing
  if (control1) print*, 'making f_pade'
  f_pade=fee+fie+fii
end subroutine

subroutine virial(n_e,n_a,f_int_a)
!in- and output
  double precision, intent(in)::n_e,n_a
  double precision, intent(out)::f_int_a

!help
  double precision bca
  
  call virkoff(bca)
  f_int_a=bca*n_a*n_e

end subroutine

!*********** helping routines ***********

subroutine virkoff(bca)
!2. virial coefficient for mue_a
  double precision, intent(out)::bca

  double precision::bea,z,z2,v,pv,qv,si,ci,fz,fn,gz,gn, vhoch
  double precision, parameter::eul=0.577215665d0
  double precision, dimension(5), parameter::&
    p=(/1.964882d-2,6.939889d-4,1.374168d-5,1.568988d-7,1.753141d-9/),&
    q=(/4.990920d-3,1.185999d-4,1.725752d-6,1.584996d-8,1.386985d-10/)
  double precision, dimension(4), parameter::&
   af=(/3.3567732d2,2.65187033d2,3.8027264d1,1.0d0/),&
   bf=(/5.7023628d2,3.22624911d2,4.0021433d1,1.0d0/),&
   ag=(/3.52018498d2,3.02757865d2,4.2242855d1,1.0d0/),&
   bg=(/1.114978885d3,4.82485984d2,4.8196927d1,1.0d0/)
  integer::i
  
  z=kappa*rpol
  z2=2.0d0*z
  if (Z2<1.0d0) then
     v=1.6d1-z2*z2
     pv=0.4395509d0
     qv=0.1315308d0
     vhoch=1.0d0
     do i=1,5
        vhoch=v*vhoch
        pv=pv+p(i)*vhoch
        qv=qv+q(i)*vhoch
     end do
     si=-pi/2.0d0+z2*pv
     ci=eul+dlog(z2)-z2*z2*qv
     bea=-z/2.0d0+(ci*dsin(z2)-si*dcos(z2))*(0.5d0+z*z/2.0d0)&
        -(z+z**3.0)*(ci*dcos(z2)+si*dsin(z2))
  else
     fz=3.8102495d1
     fn=1.57105423d2
     gz=2.1821899d1
     gn=4.49690326d2
     vhoch=1.0d0
     do i=1,4
        vhoch=vhoch*z2*z2
        fz=fz+af(i)*vhoch
        fn=fn+bf(i)*vhoch
        gz=gz+ag(i)*vhoch
        gn=gn+bg(i)*vhoch
     end do
     bea=-z/2.0d0+(0.25d0/z+z/4.0d0)*(fz/fn+gz/gn)
  end if
     bca=-4.0d0*pi*pol*bea/rpol
end subroutine

!********************* determination of n *******************************
subroutine mk_n(k,n)
!in- and output
  double precision, dimension(maxion), intent(in)::k
  double precision, dimension(maxion+2), intent(inout)::n

!helping variables
  double precision, dimension(Z_min:Z_max)::ns, hilfe
  double precision, dimension(Z_min+1:Z_max)::spinhilf
  integer i,stelle


!initialization
  if (control1) print*, 'mk_n: initialisation'
  ns(Z_min:Z_max)=n(Z_min+2:Z_max+2)
  spinhilf(Z_min+1:Z_max)=dlog(spinparameter(Z_min+1:Z_max))
  hilfe=0

!ion with maximum fraction
  if (control1) print*, 'mk_n: ion with maximum fraction'
  stelle=minion
  if (Z_max>stelle) then
     do i=minion+1,Z_max
        if (ns(i)>ns(stelle) .or. i==1) stelle=i
     end do
  end if

!calculation of n
  if (control1) print*, 'mk_n: calculation of n'
  hilfe(stelle)=0.0d0
  do i=stelle-1,Z_min,-1
     hilfe(i)=hilfe(i+1)+spinhilf(i+1)+k(i+1)
  end do
  if (stelle<Z_max) then
     do i=stelle+1,Z_max
        hilfe(i)=hilfe(i-1)-spinhilf(i)-k(i)
     end do
  end if

  if (maxval(hilfe)>5.0d2) then
!     print*, 'mk_n: warning: hilfe>500'
!     print*, 'Z_min=', Z_min, 'Z_max=', Z_max
!     print*, 'k(Z_min+1:Z_max)=', k(Z_min+1:Z_max)
     hilfe=dexp(5.0d0)
!     stop
  else
     hilfe=dexp(hilfe)
  end if

  ns(stelle)=ne/sum(i_array(minion:Z_max)*hilfe(minion:Z_max))
  ns(Z_min:Z_max)=ns(stelle)*hilfe(Z_min:Z_max)
  if (control1) print*, 'mk_n: n ready'
  n(Z_min+2:Z_max+2)=ns(Z_min:Z_max) 

  if (control1) print*, 'leaving mk_n'

end subroutine


!%%%%%%%%%%% for maxion=1 %%%%%%%%%%%%%%%%%%%%%%
subroutine mk_n0_hs(beta_k,n)
!in- and output
  double precision, dimension(maxion), intent(in)::beta_k
  double precision, dimension(maxion+2), intent(inout)::n

  n(3)=n(1) !electron number
  n(2)=spinparameter(1)*n(3)*dexp(beta_k(1)) !atoms

end subroutine

subroutine mk_n0(beta_k,n)
!Ein- und Ausgabe
  double precision, dimension(maxion), intent(in)::beta_k
  double precision, dimension(maxion+2), intent(inout)::n

!Hilfsvariablen
  double precision const,x_lb,x_ub,y_lb,y_ub,x,y
  double precision, dimension(-1:maxion):: ns
  logical steigend

  ns(-1:maxion)=n(1:maxion+2)
  ns(0)=spinparameter(1)*ns(1)*dexp(beta_k(1))

  const=bca/kbt*ns(0)

  x_lb=ns(0)*bca/kbt
  x_ub=x_lb+0.1d0*dabs(x_lb)
  y_lb=const*dexp(x_lb)-x_lb
  y_ub=const*dexp(x_ub)-x_ub

  do while (y_lb*y_ub>0.0d0)
     x=-(x_ub-x_lb)/(y_ub-y_lb)*y_lb+x_lb
     if (x<x_lb) then
        x_ub=x_lb
        x_lb=x
     end if
     if (x>x_ub) then
        x_lb=x_ub
        x_ub=x
     end if
     y_lb=const*dexp(x_lb)-x_lb
     y_ub=const*dexp(x_ub)-x_ub
  end do
  if (y_ub>=y_lb) steigend=.true.

  do while (dabs(x_ub/x_lb)>1.00001d0)
     x=(x_ub-x_lb)/(y_ub-y_lb)
     y=const*dexp(x)-x
     if (y==0.0d0) then
        x_ub=x
        x_lb=x
     else
        if (steigend) then
           if (y>0.0d0) x_ub=x
           if (y<0.0d0) x_lb=x
        else
           if (y<0.0d0) x_ub=x
           if (y>0.0d0) x_lb=x
        end if
     end if
  end do

  x=0.5d0*(x_lb+x_ub)
  ns(0)=x*kbt/bca

  n(1:maxion+2)=ns(-1:maxion)

end subroutine



end module

