!******************************************************************************
!******************************************************************************
!******************************************************************************

module constants

!******************************************************************************
!contains mathematical and physical constants in different unit systems
!
!Ref: Bronstein, Musiol, Muehlig: 
!     "Taschenbuch der Mathematik"   1. Auflage 1993
!     Verlag Harri Deutsch, Thun, Frankfurt am Main 1993
!
!     Homepage des Vereins "Freunde der Zahl PI"
!     URL: www.ast.univie.ac.at/~wasi/PI/pi_klub.html
!     Stand 01/1999
!
!     Torsten Seifert:
!     "Verschiebung von Zwei-Teilchen-Energien und Spektrallinien 
!      in dichten Plasmen"  (Doktorarbeit; Anhang)
!     W.Pieck Universitaet Rostock, September 1981
!******************************************************************************

  save

!mathematical constants
  double precision, parameter:: pi=3.14159265358979d0
  double precision, parameter:: euler=2.718281828d0   


!physical constants
  !SI-units
  double precision, parameter:: avogadro=6.022136736d23          !1/mol
  double precision, parameter:: loschmidt=2.68676323d25          !1/m^3
  double precision, parameter:: atommasse=1.660540210d-27        !kg
  double precision, parameter:: abohr=5.2917715d-11              !m
  double precision, parameter:: kb=1.38065812d-23                !J/K
  double precision, parameter:: pmasse=1.672623110d-27           !kg
  double precision, parameter:: emasse=9.109389754d-31           !kg
  double precision, parameter:: eladung=1.6021773349d-19         !As
  double precision, parameter:: gerde=9.80665d0                  !m/s^2
  double precision, parameter:: gravitation=6.6725985d-11        !m^3/kg/s^2
  double precision, parameter:: epsilon0=8.854187817d-12         !As/Vm
  double precision, parameter:: planck=6.626075540d-34           !Js
  double precision, parameter:: planckquer=planck/(2*pi)
  double precision, parameter:: erydberg=13.605698140d0*eladung  !VAs

  !in often used non-SI units
  double precision, parameter:: kbev=8.617385d-5                 !eV/K
  double precision, parameter:: erydbergev=13.605698140d0        !eV
  double precision, parameter:: planckev=4.135669212d-15         !eVs
  double precision, parameter:: planckquerev=planckev/(2*pi)

  !in atomic units
  double precision, parameter:: loschmidtat=2.68676323d25*abohr**3 
  double precision, parameter:: atommasseat=1822.89d0
  double precision, parameter:: kbat=kbev/erydbergev
  double precision, parameter:: pmasseat=1836.15270137
  double precision, parameter:: emasseat=0.5d0
  double precision, parameter:: equadratat=2.0d0
  double precision, parameter:: epsilon0at=1.0d0/(4*pi)
  double precision, parameter:: planckquerat=1.0d0
  double precision, parameter:: zeitat=4.8377d-17   !atomic unit of time

end module
