//     The function g_j_ calculates for a given ion the total angular momentum J of 
//     the ion in ground state after Hund's rules. Out of it the degeneracy 
//     factor g_j=2*J+1 is calculated.
//
//     It uses the functions groundfill, trans_el, and DTotAngMom. The purposes of these 
//     functions are as follows:
//
//     There is an array named atom in which an arbitrary elctronic configuration of an atom
//     can be stored. The function groundfill fills this array with electrons with energetic
//     lowest orbitals first.
//     The function trans_el changes this configuration with a view to the anomalies being 
//     observed within some transition elements and the Lanthanoids.
//     Anomalies for elements beyond Radonnot yet considered, so it is only usefull for Z<86 
//     Finally the function DTotAngMom calculates the double total angular momentum. 
//
//     This modularization was done, to be able to do more extensions on the electronic 
//     configuration, such as exciting an atom with the temperature.
//     
//     Bastian Holst, 03.08.2004
//

#define MAXMOM 7
#define MAXMQN 7

double g_j_(int*,int*);
void groundfill(int*,int,int*,int*);
void trans_el(int*,int,int*,int*);
int DTotAngMom(int*);

double g_j_(int *ordz,int *ion){
	if ((*ordz-*ion)<=0) return 1;
	int atom[MAXMQN][MAXMOM],maxn,maxl;
	groundfill(&atom[0][0],*ordz-*ion,&maxn,&maxl);
	trans_el(&atom[0][0],*ordz-*ion,&maxn,&maxl);
	return DTotAngMom(&atom[0][0])+1;
}

void groundfill(int *atom,int ordz,int *maxn,int *maxl){
	int l=0,n=0,left,put,i,j,ntemp;
	left=ordz;
	for (i=0;i<MAXMQN;i++)
		for (j=0;j<MAXMOM;j++) *(atom+i*MAXMOM+j)=0;
	while (left >0){
		put = 2*(2*l+1);
		if (put > left) put=left;
		*(atom+n*MAXMOM+l)=put;
		*maxn=n;*maxl=l;
		if (l==0){
			ntemp=n+1;
			n+=(1-(n+1)/2);
			l=ntemp/2;
		}else{
			n+=1;
			l-=1;
		}
		left-=put;
	}
}

void trans_el(int *atom, int ordz, int *maxn, int *maxl){
	if (*maxl=2){
		switch (*maxn){
		case 2:	if((*(atom+*maxn*MAXMOM+*maxl)-4)%5==0){
				*(atom+*maxn*MAXMOM+*maxl)+=1;
				*(atom+(*maxn+1)*MAXMOM+*maxl-2)-=1;
			}
			break;
		case 3: if ((*(atom+*maxn*MAXMOM+*maxl)>2)&&(*(atom+*maxn*MAXMOM+*maxl)<10)){
				*(atom+*maxn*MAXMOM+*maxl)+=1;
				*(atom+(*maxn+1)*MAXMOM+*maxl-2)-=1;
			}
			if (*(atom+*maxn*MAXMOM+*maxl)==9){
				*(atom+*maxn*MAXMOM+*maxl)+=1;
				*(atom+(*maxn+1)*MAXMOM+*maxl-2)-=1;
			}	
			break;
		case 4: if ((*(atom+*maxn*MAXMOM+*maxl)>6)&&(*(atom+*maxn*MAXMOM+*maxl)<10)){
				*(atom+*maxn*MAXMOM+*maxl)+=1;
				*(atom+(*maxn+1)*MAXMOM+*maxl-2)-=1;
			}
			if (*(atom+*maxn*MAXMOM+*maxl)==8){
				*(atom+*maxn*MAXMOM+*maxl)+=1;
				*(atom+(*maxn+1)*MAXMOM+*maxl-2)-=1;
			}
			break;
		}
	}
	if (*maxl=3){
		if ((*(atom+*maxn*MAXMOM+*maxl)-1)%7==0){
			*(atom+*maxn*MAXMOM+*maxl)-=1;
			*(atom+(*maxn+1)*MAXMOM+*maxl-1)+=1;
			*maxn+=1;
			*maxl-=1;
		}
	}
}

int DTotAngMom(int *atom){
	int l,n,DJ=0,DL=0,DS=0,help;
	for (n=0;n<MAXMQN;n++){
		for (l=0;l<MAXMOM;l++){
			if((*(atom+n*MAXMOM+l)>0)&&(*(atom+n*MAXMOM+l)<(2*l+1)*2)){
				if (*(atom+n*MAXMOM+l)<=2*l+1){	
					DS=*(atom+n*MAXMOM+l);
				}else{
					DS=2*(2*l+1)-*(atom+n*MAXMOM+l);
				}
				help=*(atom+n*MAXMOM+l)%(2*l+1);
				DL=help*(2*l-help+1);
				if (DJ!=0){
					if (*(atom+n*MAXMOM+l)<2*l+1){
						DJ-=DL-DS;
					}else{
						DJ+=DL+DS;
					}
				}else{	
					if (*(atom+n*MAXMOM+l)<2*l+1){
						DJ+=DL-DS;
					}else{
						DJ+=DL+DS;
					}
				}	
			}
		}
	}
	return DJ;
}

