/*  DATEI ENTHLT STANDARTFUNKTIONEN AUS " NRUTIL.C " FR PROGRAMME     */
/*  DIE MIT FUNKTIONEN AUS " NUMERICAL RECIPES " ARBEITEN               */

#ifndef __utilites
#define __utilites

#include "header.h"
#include <stdlib.h>

static void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

#define NR_END 1
#define FREE_ARG char*

TYP *vector(long nl, long nh)
/* allocate a TYP vector with subscript range v[nl..nh] */
{
	TYP *v;

	v=(TYP *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(TYP)));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl+NR_END;
}

void free_vector(TYP *v, long nl)
/* free a TYP vector allocated with vector() */
{
	free((FREE_ARG) (v+nl-NR_END));
}

#define NRANSI

void polint(TYP xa[], TYP ya[], int n, TYP x, TYP *y, TYP *dy)
{
	int i,m,ns=1;
	TYP den,dif,dift,ho,hp,w;
	TYP *c,*d;

	dif=fabs(x-xa[1]);
	c=vector(1,n);
	d=vector(1,n);
	for (i=1;i<=n;i++) {
		if ( (dift=fabs(x-xa[i])) < dif) {
			ns=i;
			dif=dift;
		}
		c[i]=ya[i];
		d[i]=ya[i];
	}
	*y=ya[ns--];
	for (m=1;m<n;m++) {
		for (i=1;i<=n-m;i++) {
			ho=xa[i]-x;
			hp=xa[i+m]-x;
			w=c[i+1]-d[i];
			if ( (den=ho-hp) == 0.0) nrerror("Error in routine polint");
			den=w/den;
			d[i]=hp*den;
			c[i]=ho*den;
		}
		*y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
	}
	free_vector(d,1);
	free_vector(c,1);
}
#undef NRANSI
/* (C) Copr. 1986-92 Numerical Recipes Software $1&. */

#endif
